/**
 * fwdmodel_dce.h
 *
 * Base class for DCE models. Provides basic functions for dealing with AIF and
 * standard parameters (T1, delay, sig0).
 *
 * Martin Craig, IBME
 *
 * Copyright (C) 2016 University of Oxford
 */

/*  Part of FSL - FMRIB's Software Library
    http://www.fmrib.ox.ac.uk/fsl
    fsl@fmrib.ox.ac.uk

    Developed at FMRIB (Oxford Centre for Functional Magnetic Resonance
    Imaging of the Brain), Department of Clinical Neurology, Oxford
    University, Oxford, UK


    LICENCE

    FMRIB Software Library, Release 6.0 (c) 2018, The University of
    Oxford (the "Software")

    The Software remains the property of the Oxford University Innovation
    ("the University").

    The Software is distributed "AS IS" under this Licence solely for
    non-commercial use in the hope that it will be useful, but in order
    that the University as a charitable foundation protects its assets for
    the benefit of its educational and research purposes, the University
    makes clear that no condition is made or to be implied, nor is any
    warranty given or to be implied, as to the accuracy of the Software,
    or that it will be suitable for any particular purpose or for use
    under any specific conditions. Furthermore, the University disclaims
    all responsibility for the use which is made of the Software. It
    further disclaims any liability for the outcomes arising from using
    the Software.

    The Licensee agrees to indemnify the University and hold the
    University harmless from and against any and all claims, damages and
    liabilities asserted by third parties (including claims for
    negligence) which arise directly or indirectly from the use of the
    Software or the sale of any products based on the Software.

    No part of the Software may be reproduced, modified, transmitted or
    transferred in any form or by any means, electronic or mechanical,
    without the express permission of the University. The permission of
    the University is not required if the said reproduction, modification,
    transmission or transference is done without financial return, the
    conditions of this Licence are imposed upon the receiver of the
    product, and all original and amended source code is included in any
    transmitted product. You may be held legally responsible for any
    copyright infringement that is caused or encouraged by your failure to
    abide by these terms and conditions.

    You are not permitted under this Licence to use this Software
    commercially. Use for which any financial return is received shall be
    defined as commercial use, and includes (1) integration of all or part
    of the source code or the Software into a product for sale or license
    by or on behalf of Licensee to third parties or (2) use of the
    Software or any derivative of it for research with the final aim of
    developing software products for sale or license to a third party or
    (3) use of the Software or any derivative of it for research with the
    final aim of developing non-software products for sale or license to a
    third party, or (4) use of the Software to provide any service to an
    external organisation for which payment is received. If you are
    interested in using the Software commercially, please contact Oxford
    University Innovation ("OUI"), the technology transfer company of the
    University, to negotiate a licence. Contact details are:
    fsl@innovation.ox.ac.uk quoting Reference Project 9564, FSL.*/
#pragma once

#include "fabber_core/fwdmodel.h"

#include "armawrap/newmat.h"

#include <string>
#include <vector>

/**
 * Base class for DCE models as they share options
 *
 * Should probably factor out AIF into a separate class one per implementation.
 * Currently have Orton (2008), Parker (2006) and user-specified (signal or concentration)
 */
class DCEFwdModel : public FwdModel
{
public:
    virtual ~DCEFwdModel()
    {
    }

    virtual std::string ModelVersion() const;
    virtual void GetOptions(std::vector<OptionSpec> &opts) const;

    virtual void Initialize(FabberRunData &rundata);
    virtual void GetParameterDefaults(std::vector<Parameter> &params) const;

    virtual void InitVoxelPosterior(MVNDist &posterior) const;

protected:
    // Mandatory DCE configuration
    double m_fa, m_tr, m_r1, m_dt;
    std::string m_aif_type;

    // Standard DCE parameters (fixed when not inferred)
    double m_delay, m_t10, m_sig0;

    // Inference flags
    bool m_infer_delay, m_infer_t10, m_infer_sig0;

    // AIF as concentration curve
    NEWMAT::ColumnVector m_aif;

    // Other options
    mutable int m_sig0_idx;
    bool m_auto_init_delay;

    // Orton AIF parameters
    double m_ab, m_ag, m_mub, m_mug;

    double SignalFromConcentration(double C, double t10, double m0) const;
    double ConcentrationFromSignal(double s, double s0, double t10, double hct) const;
    double OrtonF(double t, double a) const;
    double OrtonAIF(double t) const;
    double ParkerAIF(double t) const;
    double AIF(double t) const;
};
